/*********************************************************************
*               SEGGER MICROCONTROLLER GmbH & Co KG                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2013  SEGGER Microcontroller GmbH & Co KG         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: 3.88f                                            *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : BSP.c
Purpose : BSP for the ST STM32F40G-Eval eval board
--------  END-OF-HEADER  ---------------------------------------------
*/
#include "BSP.h"
#include "RTOS.h"
#include "stm32f4xx.h"     // Device specific header file, contains CMSIS

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/
#define GPIOA_BASE_ADDR           ((unsigned int)0x40020000)
#define GPIOA_MODER               (*(volatile unsigned int*)(GPIOA_BASE_ADDR + 0x00))
#define GPIOA_AFRL                (*(volatile unsigned int*)(GPIOA_BASE_ADDR + 0x20))
#define GPIOA_AFRH                (*(volatile unsigned int*)(GPIOA_BASE_ADDR + 0x24))

#define GPIOB_BASE_ADDR           ((unsigned int)0x40020400)
#define GPIOB_MODER               (*(volatile unsigned int*)(GPIOB_BASE_ADDR + 0x00))
#define GPIOB_AFRL                (*(volatile unsigned int*)(GPIOB_BASE_ADDR + 0x20))
#define GPIOB_AFRH                (*(volatile unsigned int*)(GPIOB_BASE_ADDR + 0x24))

#define GPIOC_BASE_ADDR           ((unsigned int)0x40020800)
#define GPIOC_MODER               (*(volatile unsigned int*)(GPIOC_BASE_ADDR + 0x00))
#define GPIOC_AFRL                (*(volatile unsigned int*)(GPIOC_BASE_ADDR + 0x20))
#define GPIOC_AFRH                (*(volatile unsigned int*)(GPIOC_BASE_ADDR + 0x24))

#define GPIOG_BASE_ADDR           ((unsigned int)0x40021800)
#define GPIOG_MODER               (*(volatile unsigned int*)(GPIOG_BASE_ADDR + 0x00))
#define GPIOG_AFRL                (*(volatile unsigned int*)(GPIOG_BASE_ADDR + 0x20))
#define GPIOG_AFRH                (*(volatile unsigned int*)(GPIOG_BASE_ADDR + 0x24))

#define GPIOH_BASE_ADDR           ((unsigned int)0x40021C00)
#define GPIOH_MODER               (*(volatile unsigned int*)(GPIOH_BASE_ADDR + 0x00))
#define GPIOH_AFRL                (*(volatile unsigned int*)(GPIOH_BASE_ADDR + 0x20))
#define GPIOH_AFRH                (*(volatile unsigned int*)(GPIOH_BASE_ADDR + 0x24))

#define GPIOI_BASE_ADDR           ((unsigned int)0x40022000)
#define GPIOI_MODER               (*(volatile unsigned int*)(GPIOI_BASE_ADDR + 0x00))
#define GPIOI_AFRL                (*(volatile unsigned int*)(GPIOI_BASE_ADDR + 0x20))
#define GPIOI_AFRH                (*(volatile unsigned int*)(GPIOI_BASE_ADDR + 0x24))

/****** LED port assignement on MB786 eval board ********************/

#define GPIOG_MODER               (*(volatile unsigned int*)(GPIOG_BASE_ADDR + 0x00))
#define GPIOG_ODR                 (*(volatile unsigned int*)(GPIOG_BASE_ADDR + 0x14))
#define GPIOG_BSRR                (*(volatile unsigned int*)(GPIOG_BASE_ADDR + 0x18))

/****** SFRs used for LED-Port **************************************/

#define RCC_BASE_ADDR             ((unsigned int)(0x40023800))
#define RCC_AHB1RSTR              (*(volatile unsigned int*)(RCC_BASE_ADDR + 0x10))
#define RCC_AHB2RSTR              (*(volatile unsigned int*)(RCC_BASE_ADDR + 0x14))
#define RCC_AHB1ENR               (*(volatile unsigned int*)(RCC_BASE_ADDR + 0x30))
#define RCC_AHB2ENR               (*(volatile unsigned int*)(RCC_BASE_ADDR + 0x34))

#define RCC_LEDPORT_RSTR          RCC_AHB1RSTR
#define RCC_LEDPORT_ENR           RCC_AHB1ENR
#define RCC_LEDPORT_BIT           (6)

/****** Assign LEDs to Ports ****************************************/

#define LED_PORT_MODER            GPIOG_MODER
#define LED_PORT_ODR              GPIOG_ODR
#define LED_PORT_BSRR             GPIOG_BSRR

#define LED0_BIT                  (6)
#define LED1_BIT                  (8)
#define LED_MASK_ALL              ((1uL << LED0_BIT) | (1uL << LED1_BIT))

/*********************************************************************
*
*       Typedefs
*
**********************************************************************
*/

typedef void USB_ISR_HANDLER  (void);
typedef void ETH_ISR_HANDLER  (void);

/*********************************************************************
*
*       Static data
*
**********************************************************************
*/

static ETH_ISR_HANDLER * _pfEthISRHandler;
static USB_ISR_HANDLER * _pfUSBISRHandler;
static USB_ISR_HANDLER * _pfUSBHISRHandler;

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       BSP_Init()
*/
void BSP_Init(void) {
  //
  // Initialize port for LEDs (sample application)
  //
  RCC_LEDPORT_ENR  &= ~(1uL << RCC_LEDPORT_BIT);
  RCC_LEDPORT_RSTR &= ~(1uL << RCC_LEDPORT_BIT);
  RCC_LEDPORT_ENR  |=  (1uL << RCC_LEDPORT_BIT);

  LED_PORT_MODER   &= ~(3uL << (LED0_BIT * 2)) | (3uL << (LED1_BIT * 2));   // Reset mode; sets port to input
  LED_PORT_MODER   |=  (1uL << (LED0_BIT * 2)) | (1uL << (LED1_BIT * 2));   // Set to output mode
  LED_PORT_BSRR     =  (0x10000uL << LED0_BIT) | (0x10000uL << LED1_BIT);   // Initially clear LEDs
}

/*********************************************************************
*
*       LED switching routines
*       LEDs STAT1 and STAT2 are switched on with high level on port lines
*/
void BSP_SetLED(int Index) {
  if (Index == 0) {
    LED_PORT_BSRR = (1uL << LED0_BIT);       // Switch on LED0
  } else if (Index == 1) {
    LED_PORT_BSRR = (1uL << LED1_BIT);       // Switch on LED1
  }
}

void BSP_ClrLED(int Index) {
  if (Index == 0) {
    LED_PORT_BSRR = (0x10000uL << LED0_BIT); // Switch off LED0
  } else if (Index == 1) {
    LED_PORT_BSRR = (0x10000uL << LED1_BIT); // Switch off LED1
  }
}

void BSP_ToggleLED(int Index) {
  if (Index == 0) {
    if ((LED_PORT_ODR & (1uL << LED0_BIT)) == 0) {  // LED is switched off
      LED_PORT_BSRR = (1uL << LED0_BIT);            // Switch on LED0
    } else {
      LED_PORT_BSRR = (0x10000uL << LED0_BIT);      // Switch off LED0
    }
  } else if (Index == 1) {
    if ((LED_PORT_ODR & (1uL << LED1_BIT)) == 0) {  // LED is switched off
      LED_PORT_BSRR = (1uL << LED1_BIT);            // Switch on LED1
    } else {
      LED_PORT_BSRR = (0x10000uL << LED1_BIT);      // Switch off LED1
    }
  }
}

/*********************************************************************
*
*       USB
*
*  Functions for USB controllers (as far as present)
*/

/****** Declare ISR handler here to avoid "no prototype" warning. They are not declared in any CMSIS header */

#ifdef __cplusplus
extern "C" {
#endif
void OTG_FS_IRQHandler(void);
void OTG_HS_IRQHandler(void);
#ifdef __cplusplus
}
#endif

/*********************************************************************
*
*       OTG_FS_IRQHandler
*/
void OTG_FS_IRQHandler(void) {
  OS_EnterInterrupt(); // Inform embOS that interrupt code is running
  if (_pfUSBISRHandler) {
    (_pfUSBISRHandler)();
  }
  if (_pfUSBHISRHandler) {
    (_pfUSBHISRHandler)();
  }
  OS_LeaveInterrupt(); // Inform embOS that interrupt code is left
}

/*********************************************************************
*
*       OTG_HS_IRQHandler
*/
void OTG_HS_IRQHandler(void) {
  OS_EnterInterrupt(); // Inform embOS that interrupt code is running
  if (_pfUSBISRHandler) {
    (_pfUSBISRHandler)();
  }
  if (_pfUSBHISRHandler) {
    (_pfUSBHISRHandler)();
  }
  OS_LeaveInterrupt(); // Inform embOS that interrupt code is left
}

/*********************************************************************
*
*       BSP_USB_InstallISR_Ex()
*/
void BSP_USB_InstallISR_Ex(int ISRIndex, void (*pfISR)(void), int Prio){
  _pfUSBISRHandler = pfISR;

  NVIC_SetPriority((IRQn_Type)ISRIndex, (uint32_t)Prio);
  NVIC_EnableIRQ((IRQn_Type)ISRIndex);
}

/*********************************************************************
*
*       ETH
*
*  Functions for ethernet controllers (as far as present)
*/

/****** Declare ISR handler here to avoid "no prototype" warning. They are not declared in any CMSIS header */

#ifdef __cplusplus
extern "C" {
#endif
void ETH_IRQHandler(void);
#ifdef __cplusplus
}
#endif


/*********************************************************************
*
*       ETH_IRQHandler
*/
void ETH_IRQHandler(void) {
  OS_EnterInterrupt(); // Inform embOS that interrupt code is running
  if (_pfEthISRHandler) {
    (_pfEthISRHandler)();
  }
  OS_LeaveInterrupt(); // Inform embOS that interrupt code is left
}

/*********************************************************************
*
*       BSP_ETH_InstallISR_Ex()
*/
void BSP_ETH_InstallISR_Ex(int ISRIndex, void (*pfISR)(void), int Prio){
  (void)ISRIndex;  /* Not implemented, avoid warning */
  (void)pfISR;     /* Not implemented, avoid warning */
  (void)Prio;      /* Not implemented, avoid warning */
}

  /*********************************************************************
*
*       BSP_ETH_InstallISR()
*/
void BSP_ETH_InstallISR (void (*pfISR)(void)){
  _pfEthISRHandler = pfISR;
   NVIC_SetPriority(ETH_IRQn, (1 << __NVIC_PRIO_BITS) - 2);
   NVIC_EnableIRQ(ETH_IRQn);
}

/*********************************************************************
*
*       BSP_ETH_Init()
*
*  Function description
*    This routine is called from the Ethernet driver to initialize
*    port pins and enable clocks.
*
*  Note:
*    Port         Pin function
*      PA1          ETH_MII_RX_CLK / ETH_RMII _REF_CLK
*      PA2          ETH_MDIO
*      PA7          ETH_MII_RX_DV / ETH_RMII _CRS_DV
*      PB8          ETH_MII_TXD3
*      PC1          ETH_MDC
*      PC2          ETH_MII_TXD2
*      PC3          ETH_MII_TX_CLK
*      PC4          ETH_MII_RXD0 / ETH_RMII_RXD0
*      PC5          ETH_MII_RXD1/ ETH _RMII_RXD1
*      PG11         ETH_MII_TX_EN / ETH _RMII_TX_EN
*      PG13         ETH_MII_TXD0 / ETH _RMII_TXD0
*      PG14         ETH_MII_TXD1 / ETH _RMII_TXD1
*      PH2          ETH_MII_CRS
*      PH3          ETH_MII_COL
*      PH6          ETH_MII_RXD2
*      PH7          ETH_MII_RXD3
*      PI10         ETH_MII_RX_ER
*/
void BSP_ETH_Init (unsigned Unit){
  unsigned int v;

  (void)Unit;   // Multiple Units not supported, avoid warning   
  //
  // Enable I/O clocks
  //
  RCC_AHB1ENR |= 0
               | (1 <<  8)  // GPIOAEN: IO port I clock enable
               | (1 <<  7)  // GPIOAEN: IO port H clock enable
               | (1 <<  6)  // GPIOAEN: IO port G clock enable
               | (1 <<  2)  // GPIOAEN: IO port C clock enable
               | (1 <<  1)  // GPIOAEN: IO port B clock enable
               | (1 <<  0)  // GPIOAEN: IO port A clock enable
               ;
  //
  // Set PA1 (ETH_MII_RX_CLK/ETH_RMII_REF_CLK) as alternate function
  //
  v           = GPIOA_MODER;
  v          &= ~(0x3uL << (2 * 1));
  v          |=  (0x2uL << (2 * 1));
  GPIOA_MODER = v;
  v           = GPIOA_AFRL;
  v          &= ~(0xFuL << (4 * 1));
  v          |=  (0xBuL << (4 * 1));
  GPIOA_AFRL  = v;
  //
  // Set PA2 (ETH_MDIO) as alternate function
  //
  v           = GPIOA_MODER;
  v          &= ~(0x3uL << (2 * 2));
  v          |=  (0x2uL << (2 * 2));
  GPIOA_MODER = v;
  v           = GPIOA_AFRL;
  v          &= ~(0xFuL << (4 * 2));
  v          |=  (0xBuL << (4 * 2));
  GPIOA_AFRL  = v;
  //
  // Set PA7 (ETH_MII_RX_DV/ETH_RMII_CRS_DV) as alternate function
  //
  v           = GPIOA_MODER;
  v          &= ~(0x3uL << (2 * 7));
  v          |=  (0x2uL << (2 * 7));
  GPIOA_MODER = v;
  v           = GPIOA_AFRL;
  v          &= ~(0xFuL << (4 * 7));
  v          |=  (0xBuL << (4 * 7));
  GPIOA_AFRL  = v;
  //
  // Set PB8 (ETH_MII_TXD3) as alternate function
  //
  v           = GPIOB_MODER;
  v          &= ~(0x3uL << (2 * 8));
  v          |=  (0x2uL << (2 * 8));
  GPIOB_MODER = v;
  v           = GPIOB_AFRH;
  v          &= ~(0xFuL << (4 * 0));
  v          |=  (0xBuL << (4 * 0));
  GPIOB_AFRH  = v;
  //
  // Set PC1 (ETH_MDC) as alternate function
  //
  v           = GPIOC_MODER;
  v          &= ~(0x3uL << (2 * 1));
  v          |=  (0x2uL << (2 * 1));
  GPIOC_MODER = v;
  v           = GPIOC_AFRL;
  v          &= ~(0xFuL << (4 * 1));
  v          |=  (0xBuL << (4 * 1));
  GPIOC_AFRL  = v;
  //
  // Set PC2 (ETH_MII_TXD2) as alternate function
  //
  v           = GPIOC_MODER;
  v          &= ~(0x3uL << (2 * 2));
  v          |=  (0x2uL << (2 * 2));
  GPIOC_MODER = v;
  v           = GPIOC_AFRL;
  v          &= ~(0xFuL << (4 * 2));
  v          |=  (0xBuL << (4 * 2));
  GPIOC_AFRL  = v;
  //
  // Set PC3 (ETH_MII_TX_CLK/ETH_RMII_TX_CLK) as alternate function
  //
  v           = GPIOC_MODER;
  v          &= ~(0x3uL << (2 * 3));
  v          |=  (0x2uL << (2 * 3));
  GPIOC_MODER = v;
  v           = GPIOC_AFRL;
  v          &= ~(0xFuL << (4 * 3));
  v          |=  (0xBuL << (4 * 3));
  GPIOC_AFRL  = v;
  //
  // Set PC4 (ETH_MII_RXD0 / ETH_RMII_RXD0) as alternate function
  //
  v           = GPIOC_MODER;
  v          &= ~(0x3uL << (2 * 4));
  v          |=  (0x2uL << (2 * 4));
  GPIOC_MODER = v;
  v           = GPIOC_AFRL;
  v          &= ~(0xFuL << (4 * 4));
  v          |=  (0xBuL << (4 * 4));
  GPIOC_AFRL  = v;
  //
  // Set PC5 (ETH_MII_RXD1/ ETH_RMII_RXD1) as alternate function
  //
  v           = GPIOC_MODER;
  v          &= ~(0x3uL << (2 * 5));
  v          |=  (0x2uL << (2 * 5));
  GPIOC_MODER = v;
  v           = GPIOC_AFRL;
  v          &= ~(0xFuL << (4 * 5));
  v          |=  (0xBuL << (4 * 5));
  GPIOC_AFRL  = v;
  //
  // Set PG11 (ETH_MII_TX_EN/ETH_RMII_TX_EN) as alternate function
  //
  v           = GPIOG_MODER;
  v          &= ~(0x3uL << (2 * 11));
  v          |=  (0x2uL << (2 * 11));
  GPIOG_MODER = v;
  v           = GPIOG_AFRH;
  v          &= ~(0xFuL << (4 * 3));
  v          |=  (0xBuL << (4 * 3));
  GPIOG_AFRH  = v;
  //
  // Set PG13 (ETH_MII_TXD0/ETH_RMII_TXD0) as alternate function
  //
  v           = GPIOG_MODER;
  v          &= ~(0x3uL << (2 * 13));
  v          |=  (0x2uL << (2 * 13));
  GPIOG_MODER = v;
  v           = GPIOG_AFRH;
  v          &= ~(0xFuL << (4 * 5));
  v          |=  (0xBuL << (4 * 5));
  GPIOG_AFRH  = v;
  //
  // Set PG14 (ETH_MII_TXD1/ETH_RMII_TXD1) as alternate function
  //
  v           = GPIOG_MODER;
  v          &= ~(0x3uL << (2 * 14));
  v          |=  (0x2uL << (2 * 14));
  GPIOG_MODER = v;
  v           = GPIOG_AFRH;
  v          &= ~(0xFuL << (4 * 6));
  v          |=  (0xBuL << (4 * 6));
  GPIOG_AFRH  = v;
  //
  // Set PH2 (ETH_MII_CRS) as alternate function
  //
  v           = GPIOH_MODER;
  v          &= ~(0x3uL << (2 * 2));
  v          |=  (0x2uL << (2 * 2));
  GPIOH_MODER = v;
  v           = GPIOH_AFRL;
  v          &= ~(0xFuL << (4 * 2));
  v          |=  (0xBuL << (4 * 2));
  GPIOH_AFRL  = v;
  //
  // Set PH3 (ETH_MII_COL) as alternate function
  //
  v           = GPIOH_MODER;
  v          &= ~(0x3uL << (2 * 3));
  v          |=  (0x2uL << (2 * 3));
  GPIOH_MODER = v;
  v           = GPIOH_AFRL;
  v          &= ~(0xFuL << (4 * 3));
  v          |=  (0xBuL << (4 * 3));
  GPIOH_AFRL  = v;
  //
  // Set PH6 (ETH_MII_RXD2) as alternate function
  //
  v           = GPIOH_MODER;
  v          &= ~(0x3uL << (2 * 6));
  v          |=  (0x2uL << (2 * 6));
  GPIOH_MODER = v;
  v           = GPIOH_AFRL;
  v          &= ~(0xFuL << (4 * 6));
  v          |=  (0xBuL << (4 * 6));
  GPIOH_AFRL  = v;
  //
  // Set PH7 (ETH_MII_RXD3) as alternate function
  //
  v           = GPIOH_MODER;
  v          &= ~(0x3uL << (2 * 7));
  v          |=  (0x2uL << (2 * 7));
  GPIOH_MODER = v;
  v           = GPIOH_AFRL;
  v          &= ~(0xFuL << (4 * 7));
  v          |=  (0xBuL << (4 * 7));
  GPIOH_AFRL  = v;
  //
  // Set PI10 (ETH_MII_RX_ER) as alternate function
  //
  v           = GPIOI_MODER;
  v          &= ~(0x3uL << (2 * 10));
  v          |=  (0x2uL << (2 * 10));
  GPIOI_MODER = v;
  v           = GPIOI_AFRH;
  v          &= ~(0xFuL << (4 * 2));
  v          |=  (0xBuL << (4 * 2));
  GPIOI_AFRH  = v;
}

/****** End of file *************************************************/
